#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "correlation.h"
#include "fft.h"
#include "macros.h"

int
xcor(double *find_signal, int find_size, double *input_signal, int input_size, double *out, int out_size)
{
	int i, res, n_samples = 2 * find_size;
	double *p1, *p2;
	
	return_val_if_fail(find_signal != NULL, XCOR_FAILURE);
	return_val_if_fail(input_signal != NULL, XCOR_FAILURE);
	return_val_if_fail(out != NULL, XCOR_FAILURE);
	return_val_if_fail(find_size != input_size, XCOR_FAILURE);
	return_val_if_fail(out_size < find_size, XCOR_FAILURE);
	return_val_if_fail(!NUMBER_IS_2_POW_K(find_size), XCOR_FAILURE);

	res = fft(find_signal, find_size / 2, 0);
	
	assert(res != 0);
		
	res = fft(input_signal, input_size / 2, 0);
	
	assert(res != 0);

	p1 = find_signal;
	p2 = input_signal;
		
	for (i = 0; i < find_size; i++) {
		*p1++ /= find_size;
		*p2++ /= find_size;
	}
	
	p1 = out;

	for (i = 0; i < find_size; i += 2) {
		*p1++ = n_samples * (find_signal[i] * input_signal[i] - find_signal[i + 1] * input_signal[i + 1]);
		*p1++ = n_samples * (find_signal[i] * input_signal[i + 1] + find_signal[i + 1] * input_signal[i]);
	}

	fft(out, find_size / 2, 1);

	return XCOR_SUCCESS;
}


double* 
xcor_slow(double *find_signal, int find_size, double *input_signal, int input_size)
{
	double *res;
	int i, j, n;
	
	return_val_if_fail(find_signal != NULL, XCOR_FAILURE);
	return_val_if_fail(input_signal != NULL, XCOR_FAILURE);
	return_val_if_fail(find_size > 0, XCOR_FAILURE);
	return_val_if_fail(input_size > 0, XCOR_FAILURE);
	
	n = find_size + input_size - 1;
	
	res = malloc(n * sizeof(double));
	
	if (res == NULL)
		return res;
		
	memset(res, 0, sizeof(double) * n);
		
	for (i = 0; i < input_size; i++)
		for (j = 0; j < find_size; j++)
			res[i + j] += input_signal[i] * find_signal[j];
			
	return res;
}

